/*!
 * ContentTranslation Tools
 * A tool that allows editors to translate pages from one language
 * to another with the help of machine translation and other translation tools
 *
 * @copyright See AUTHORS.txt
 * @license GPL-2.0-or-later
 */
( function () {
	'use strict';

	/**
	 * Handles the jump between category counter and listing.
	 *
	 * @param {Event} e The click event
	 * @return {boolean} false
	 */
	function jump( e ) {
		var id = e.data.id,
			url = location.href;

		location.href = '#' + id;
		history.replaceState( null, null, url );

		return false;
	}

	function languageProps( code ) {
		return {
			lang: code,
			dir: $.uls.data.getDir( code )
		};
	}

	/**
	 * Get an element that adds space between the different parts
	 * of the category tag.
	 * This is needed because adding this space using CSS can give
	 * an incorrect result when RTL is involved:
	 * e.g. margin-left will not be flipped correctly if
	 * the target language is French, but the UI language is Hebrew.
	 *
	 * @return {string}
	 */
	function getCategorySpacer() {
		return $( '<span>' ).addClass( 'cx-category__spacer' );
	}

	/**
	 * CXCategoryCounter Class
	 *
	 * @class
	 * @param {string} language The language for the counter
	 * @param {CXCategoryTool} categoryTool The CXCategoryTool
	 */
	function CXCategoryCounter( language, categoryTool ) {
		this.language = language;
		this.categoryTool = categoryTool;
		this.$view = null;
		this.init();
	}

	/**
	 * Initializes the category counter.
	 */
	CXCategoryCounter.prototype.init = function () {
		var count = 0;

		this.$view = this.getView();

		if ( this.language === mw.cx.sourceLanguage &&
			this.categoryTool.categories.source !== null
		) {
			count = Object.keys( this.categoryTool.categories.source ).length;
		} else if ( this.language === mw.cx.targetLanguage &&
			this.categoryTool.categories.target !== null
		) {
			count = Object.keys( this.categoryTool.categories.target ).length;
		}

		this.update( count );
	};

	/**
	 * Retrieves the view for the category counter.
	 *
	 * @return {jQuery}
	 */
	CXCategoryCounter.prototype.getView = function () {
		var $view, $button, $icon, $count;

		$view = $( '<div>' )
			// The container is in a content column, but it has UI strings,
			// so the container is in the user language.
			.prop( languageProps( mw.config.get( 'wgUserLanguage' ) ) )
			.addClass( 'cx-category-widget cx-category-counter' );

		$button = $( '<button>' )
			.addClass( 'cx-category-counterbutton mw-ui-button mw-ui-quiet' );

		if ( this.language === mw.cx.sourceLanguage ) {
			$button.on( 'click', {
				id: 'cx-category-listing-anchor--source'
			}, jump );
		} else {
			$button.on( 'click', {
				id: 'cx-category-listing-anchor--translation'
			}, jump );
		}

		$icon = $( '<span>' )
			.addClass( 'cx-category-icon' );

		$count = $( '<span>' )
			.addClass( 'cx-category-count' );

		$button.append( $icon, $count );
		$view
			.append( $button )
			.hide();

		return $view;
	};

	/**
	 * Updates the count shown in the category counter.
	 *
	 * @param {number} count The category count
	 */
	CXCategoryCounter.prototype.update = function ( count ) {
		var text;

		count = mw.language.convertNumber( count );
		text = mw.msg( 'cx-tools-categories-count-message', count );
		this.$view.find( 'span.cx-category-count' ).text( text );
	};

	/**
	 * CXCategoryListing Class
	 *
	 * @class
	 * @param {string} language The language for the counter
	 * @param {CXCategoryTool} categoryTool The CXCategoryTool
	 */
	function CXCategoryListing( language, categoryTool ) {
		this.language = language;
		this.categoryTool = categoryTool;
		this.$view = null;
		this.init();
	}

	/**
	 * Initializes the category listing.
	 */
	CXCategoryListing.prototype.init = function () {
		this.$view = this.getView();

		if ( this.language === mw.cx.sourceLanguage &&
			this.categoryTool.categories.source !== null
		) {
			if ( Object.keys( this.categoryTool.categories.source ).length > 0 ) {
				this.addCategories( this.categoryTool.categories.source );
				this.$view
					.find( '.cx-category-categorylist' )
					.show()
					.end()
					.find( '.cx-category-nocategories' )
					.hide()
					.end();
			} else {
				this.$view
					.find( '.cx-category-categorylist' )
					.hide()
					.end()
					.find( '.cx-category-nocategories' )
					.show()
					.end();
			}
		} else if ( this.language === mw.cx.targetLanguage &&
			this.categoryTool.categories.target !== null
		) {
			if ( Object.keys( this.categoryTool.categories.target ).length > 0 ) {
				this.addCategories( this.categoryTool.categories.target );
				this.$view
					.find( '.cx-category-categorylist' )
					.show()
					.end()
					.find( '.cx-category-nocategories' )
					.hide()
					.end();
			} else {
				this.$view
					.find( '.cx-category-categorylist' )
					.hide()
					.end()
					.find( '.cx-category-nocategories' )
					.show()
					.end();
			}
		}
	};

	/**
	 * Handles click events on source list items.
	 *
	 * @param {Event} e The click event
	 */
	function sourceClickHandler( e ) {
		var $listItem, categoryId, categoryTool, title, count,
			hasOwn = Object.prototype.hasOwnProperty;

		$listItem = $( this );
		categoryId = $listItem.attr( 'cx-category-id' );
		categoryTool = e.data.tool;

		if ( hasOwn.call( categoryTool.categories.adapted, categoryId ) &&
			!hasOwn.call( categoryTool.categories.target, categoryId )
		) {
			title = categoryTool.categories.adapted[ categoryId ];
			categoryTool.categories.target[ categoryId ] = title;
			count = Object.keys( categoryTool.categories.target ).length;
			categoryTool.widgets.target.listing.addCategory( categoryId, title );

			$( '.cx-category--translation[cx-category-id="' + categoryId + '"]' )
				.addClass( 'cx-category-highlight' );

			categoryTool.widgets.target.counter.update( count );
			categoryTool.widgets.target.listing.$view
				.find( '.cx-category-categorylist' )
				.show()
				.end()
				.find( '.cx-category-nocategories' )
				.hide()
				.end();
		}
	}

	/**
	 * Handles click events on the remove span inside target list items.
	 *
	 * @param {Event} e The click event
	 */
	function targetClickHandler( e ) {
		var $remove, $listItem, categoryId, categoryTool, count;

		$remove = $( this );
		$listItem = $remove.parent();
		categoryId = $listItem.attr( 'cx-category-id' );
		categoryTool = e.data.tool;

		$listItem.remove();
		delete categoryTool.categories.target[ categoryId ];
		count = Object.keys( categoryTool.categories.target ).length;

		$( '.cx-category--source[cx-category-id="' + categoryId + '"]' )
			.removeClass( 'cx-category-highlight' );

		categoryTool.widgets.target.counter.update( count );
		if ( count === 0 ) {
			categoryTool.widgets.target.listing.$view
				.find( '.cx-category-categorylist' )
				.hide()
				.end()
				.find( '.cx-category-nocategories' )
				.show()
				.end();
		}
	}

	/**
	 * Highlights the category and connected category list items.
	 */
	function highlightCategory() {
		var categoryId,
			$category = $( this );

		// eslint-disable-next-line no-jquery/no-class-state
		if ( $category.hasClass( 'cx-category-disabled' ) ) {
			return;
		}

		categoryId = $category.attr( 'cx-category-id' );

		$( '[cx-category-id="' + categoryId + '"]' )
			.addClass( 'cx-category-highlight' );
	}

	/**
	 * Remove the highlight on the category and connected category list items.
	 */
	function removeCategoryHighlight() {
		var categoryId,
			$category = $( this );

		// eslint-disable-next-line no-jquery/no-class-state
		if ( $category.hasClass( 'cx-category-disabled' ) ) {
			return;
		}

		categoryId = $category.attr( 'cx-category-id' );

		$( '[cx-category-id="' + categoryId + '"]' )
			.removeClass( 'cx-category-highlight' );
	}

	/**
	 * Gets the view for the category listing.
	 *
	 * @return {jQuery}
	 */
	CXCategoryListing.prototype.getView = function () {
		var categoryTool, $view, $icon, $categoryList, $anchor, $noCategories;

		categoryTool = this.categoryTool;

		$view = $( '<div>' )
			// The container may show UI messages, so it's in the user language.
			// The categories names are in the language of the column's content.
			.prop( languageProps( mw.config.get( 'wgUserLanguage' ) ) )
			.addClass( 'cx-category-widget cx-category-listing' );
		$icon = $( '<span>' ).addClass( 'cx-category-icon' );
		$categoryList = $( '<ul>' )
			.prop( languageProps( this.language ) )
			.addClass( 'cx-category-categorylist' );
		$anchor = $( '<a>' );

		if ( this.language === mw.cx.sourceLanguage ) {
			$anchor.prop( 'id', 'cx-category-listing-anchor--source' );
			$categoryList.on( 'click', '.cx-category--source', {
				tool: categoryTool
			}, sourceClickHandler );
		} else {
			$anchor.prop( 'id', 'cx-category-listing-anchor--translation' );
			$categoryList.on( 'click', '.cx-category-remove', {
				tool: categoryTool
			}, targetClickHandler );
		}

		$categoryList
			.on( 'mouseover', '.cx-category', highlightCategory )
			.on( 'mouseout', '.cx-category', removeCategoryHighlight )
			.hide();

		$noCategories = $( '<span>' )
			.text( mw.msg( 'cx-tools-categories-count-message', 0 ) )
			.addClass( 'cx-category-nocategories' )
			.hide();

		$view.append( $anchor, $icon, $noCategories, $categoryList )
			.hide();

		return $view;
	};

	/**
	 * Adds categories to the category listing.
	 *
	 * @param {Object} categories A key value object with ids and titles
	 * @param {boolean} clear A flag to clear existing categories
	 */
	CXCategoryListing.prototype.addCategories = function ( categories, clear ) {
		var $categoryList, keys, i;

		if ( !categories ) {
			return;
		}

		clear = typeof clear !== 'boolean' ? false : clear;

		$categoryList = this.$view.find( '.cx-category-categorylist' );
		if ( clear ) {
			$categoryList.empty();
		}

		keys = Object.keys( categories ).sort();
		for ( i = 0; i < keys.length; i++ ) {
			this.addCategory( keys[ i ], categories[ keys[ i ] ] );
		}
	};

	/**
	 * Creates a source category list item.
	 *
	 * @param {string} id The category id
	 * @param {string} title The category title
	 * @return {jQuery}
	 */
	function createSourceCategoryListItem( id, title ) {
		var extract, $listItem;

		// mw.Title cannot be used because of
		// https://phabricator.wikimedia.org/T106644
		extract = title.match( /^.+?:(.*)$/ );
		$listItem = $( '<li>' )
			.addClass( 'cx-category cx-category--source' )
			.attr( 'cx-category-id', id )
			.text( extract[ 1 ] );

		return $listItem;
	}

	/**
	 * Creates a target category list item.
	 *
	 * @param {string} id The category id
	 * @param {string} title The category title
	 * @return {jQuery}
	 */
	function createTargetCategoryListItem( id, title ) {
		var extract, $categoryRemove, $listItem;

		$categoryRemove = $( '<span>' )
			.addClass( 'cx-category-remove' );

		// mw.Title cannot be used because of
		// https://phabricator.wikimedia.org/T106644
		extract = title.match( /^.+?:(.*)$/ );
		$listItem = $( '<li>' )
			.addClass( 'cx-category cx-category--translation' )
			.attr( 'cx-category-id', id )
			.text( extract[ 1 ] )
			.append( getCategorySpacer(), $categoryRemove );

		return $listItem;
	}

	/**
	 * Ads a category to the category list.
	 *
	 * @param {string} id The category id
	 * @param {string} title The category title
	 */
	CXCategoryListing.prototype.addCategory = function ( id, title ) {
		var $categoryList, $categoryListItem;

		$categoryList = this.$view.find( '.cx-category-categorylist' );

		if ( this.language === mw.cx.sourceLanguage ) {
			$categoryListItem = createSourceCategoryListItem( id, title );

			if ( !this.categoryTool.categories.adapted[ id ] ) {
				$categoryListItem.addClass( 'cx-category-disabled' );
			}
		} else {
			$categoryListItem = createTargetCategoryListItem( id, title );
		}

		$categoryList.append( $categoryListItem );
	};

	/**
	 * CX Category Tool Class.
	 *
	 * @class CXCategoryTool
	 * @param {mw.cx.SiteMapper} siteMapper
	 */
	function CXCategoryTool( siteMapper ) {
		this.categories = {
			source: null,
			target: null,
			adapted: null
		};

		this.widgets = {};
		this.siteMapper = siteMapper;
	}

	/**
	 * Creates id of specified width from number using the padding character.
	 *
	 * @param {number} number The number to make the id from
	 * @param {number} width The desired width of the id
	 * @param {string} character The character to use for padding
	 * @return {string}
	 */
	function makeCategoryId( number, width, character ) {
		var output;

		character = character || '0';
		number = String( number );
		output = number.length >= width ?
			number :
			new Array( width - number.length + 1 ).join( character ) + number;

		return 'cxCategory' + output;
	}

	/**
	 * Retrieves categories for the source article.
	 *
	 * @param {string} title The article title
	 * @param {string} language The article language
	 * @return {jQuery.Promise}
	 */
	CXCategoryTool.prototype.getArticleCategories = function ( title, language ) {
		var categoryTool,
			deferred = $.Deferred(),
			categories = {};

		if ( this.categories.source !== null ) {
			deferred.resolve( this.categories.source );
		}

		categoryTool = this;

		this.siteMapper.getApi( language ).get( {
			formatversion: 2,
			action: 'query',
			prop: 'categories',
			clshow: '!hidden',
			cllimit: 100,
			titles: title
		} ).done( function ( response ) {
			var categoriesArray = response.query.pages[ 0 ].categories || [];

			categoriesArray.forEach( function ( object, i ) {
				var categoryId;

				categoryId = makeCategoryId( i, 3, '0' );
				categories[ categoryId ] = object.title;
			} );

			categoryTool.categories.source = categories;
			deferred.resolve( categories );
		} ).fail( function ( error ) {
			mw.log( '[CX] Error while retrieving source categories ' + error );
			categoryTool.categories.source = categories;
			deferred.resolve( categories );
		} );

		return deferred.promise();
	};

	/**
	 * Adapts a set of categories.
	 *
	 * @param {Object} categories A key value listing of ids and titles
	 * @param {string} language The language for adaptation
	 * @return {jQuery.Promise}
	 */
	CXCategoryTool.prototype.adaptCategories = function ( categories, language ) {
		var categoryTool,
			categoryTitles,
			inverted,
			categoryId,
			deferred = $.Deferred();

		if ( this.categories.adapted !== null ) {
			deferred.resolve( this.categories.adapted );
		}

		categoryTool = this;

		categoryTitles = [];
		inverted = {};
		for ( categoryId in categories ) {
			categoryTitles.push( categories[ categoryId ] );
			if ( Object.prototype.hasOwnProperty.call( categories, categoryId ) ) {
				inverted[ categories[ categoryId ].replace( /[:\s]/g, '-' ) ] = categoryId;
			}
		}

		this.siteMapper.getApi( mw.cx.sourceLanguage ).post( {
			action: 'query',
			titles: categoryTitles,
			prop: 'langlinks',
			lllang: this.siteMapper.getWikiDomainCode( language ),
			lllimit: 100,
			redirects: true
		} ).done( function ( response ) {
			var redirects,
				adaptedCategories = {};

			if ( response.query ) {
				redirects = response.query.redirects || {};

				Object.keys( response.query.pages ).forEach( function ( pageId ) {
					var i, key, title,
						page = response.query.pages[ pageId ];

					for ( i in redirects ) {
						if ( redirects[ i ].to === page.title ) {
							key = redirects[ i ].from;
							break;
						}
					}

					if ( !key ) {
						key = page.title;
					}

					title = key.replace( /[:\s]/g, '-' );

					if ( title && page.langlinks ) {
						adaptedCategories[ inverted[ title ] ] = page.langlinks &&
							page.langlinks[ 0 ][ '*' ];
					}
				} );
			}

			categoryTool.categories.adapted = adaptedCategories;

			if ( categoryTool.categories.target === null ) {
				categoryTool.categories.target = $.extend( {}, adaptedCategories );
			}

			deferred.resolve( adaptedCategories );
		} ).fail( function ( error ) {
			mw.log( '[CX] Error adapting categories ' + error );
			deferred.resolve( {} );
		} );

		return deferred.promise();
	};

	/**
	 * Retrieves the categories.
	 *
	 * @return {jQuery.Promise}
	 */
	CXCategoryTool.prototype.getCategories = function () {
		var categoryTool = this,
			deferred = $.Deferred();

		this.getArticleCategories( mw.cx.sourceTitle, mw.cx.sourceLanguage )
			.done( function ( categories ) {
				categoryTool.adaptCategories( categories, mw.cx.targetLanguage )
					.done( function () {
						deferred.resolve( categoryTool.categories );
					} );
			} );

		return deferred.promise();
	};

	/**
	 * Initalizes UI widgets and attaches them to DOM.
	 *
	 * @param {string} column The name of the column to initialize widgets for
	 */
	CXCategoryTool.prototype.initializeWidgets = function ( column ) {
		if ( column === 'source' ) {
			this.widgets.source = {};
			this.widgets.source.counter = new CXCategoryCounter( mw.cx.sourceLanguage, this );
			this.widgets.source.listing = new CXCategoryListing( mw.cx.sourceLanguage, this );
		}

		if ( column === 'translation' ) {
			this.widgets.target = {};
			this.widgets.target.counter = new CXCategoryCounter( mw.cx.targetLanguage, this );
			this.widgets.target.listing = new CXCategoryListing( mw.cx.targetLanguage, this );
		}
	};

	/**
	 * Attaches UI widgets to DOM.
	 *
	 * @param {string} column The name of the column to attach widgets to
	 */
	CXCategoryTool.prototype.attachWidgets = function ( column ) {
		var categoryTool = this;

		// Load widgets for the source column
		if ( column === 'source' ) {
			$( 'div.cx-column--source > div.cx-column__sub-heading' )
				.after( categoryTool.widgets.source.counter.$view );
			$( 'div.cx-column--source > div.cx-column__content' )
				.after( categoryTool.widgets.source.listing.$view );
		}

		// Load widgets for the translation column
		if ( column === 'translation' ) {
			$( 'div.cx-column--translation > div.cx-column__sub-heading' )
				.after( categoryTool.widgets.target.counter.$view );
			$( 'div.cx-column--translation > div.cx-column__content' )
				.after( categoryTool.widgets.target.listing.$view );
		}
	};

	/**
	 * Shows UI widgets.
	 *
	 * @param {string} column The name of the column to show widgets for
	 */
	CXCategoryTool.prototype.showWidgets = function ( column ) {
		if ( column === 'source' ) {
			this.widgets.source.counter.$view.show();
			this.widgets.source.listing.$view.show();
		}

		if ( column === 'translation' ) {
			this.widgets.target.counter.$view.show();
			this.widgets.target.listing.$view.show();
		}
	};

	// Expose the CXCategoryTool class for unit testing
	if ( typeof QUnit !== 'undefined' ) {
		mw.cx.CategoryTool = CXCategoryTool;
	}

	$( function () {
		var sitemapper;

		sitemapper = new mw.cx.SiteMapper();

		// The module will be loaded in the context of Unit tests too, there
		// source language may not set. Check and abort if that is the case.
		if ( !mw.cx.sourceLanguage ) {
			return;
		}

		// Expose the CXCategoryTool (required by publishing)
		mw.cx.categoryTool = new CXCategoryTool( sitemapper );
		mw.cx.categoryTool.getCategories().done( function ( categories ) {
			mw.hook( 'mw.cx.source.loaded' ).add( function () {
				mw.cx.categoryTool.initializeWidgets( 'source' );
				mw.cx.categoryTool.initializeWidgets( 'translation' );

				if ( Object.keys( categories.source ).length > 0 ) {
					mw.cx.categoryTool.attachWidgets( 'source' );
					mw.cx.categoryTool.showWidgets( 'source' );
					window.setTimeout( function () {
						mw.cx.categoryTool.attachWidgets( 'translation' );
						mw.cx.categoryTool.showWidgets( 'translation' );
					}, 2000 );
				}
			} );
		} );
	} );
}() );
